<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Controller;

use Dedi\SyliusQuotePlugin\Context\QuoteContextInterface;
use Dedi\SyliusQuotePlugin\Entity\QuoteItemInterface;
use Dedi\SyliusQuotePlugin\Enum\QuoteActionEnum;
use Dedi\SyliusQuotePlugin\Factory\AddToQuoteCommandFactoryInterface;
use Dedi\SyliusQuotePlugin\Modifier\QuoteModifierInterface;
use Doctrine\ORM\EntityManagerInterface;
use FOS\RestBundle\View\View;
use Sylius\Bundle\ResourceBundle\Controller\ResourceController;
use Symfony\Component\Form\FormFactoryInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\HttpException;

class QuoteItemController extends ResourceController
{
    public function addItemAction(
        Request $request,
        QuoteContextInterface $quoteContext,
        FormFactoryInterface $formFactory,
        AddToQuoteCommandFactoryInterface $addToQuoteCommandFactory,
        QuoteModifierInterface $quoteModifier,
        EntityManagerInterface $entityManager
    ): Response {
        $configuration = $this->requestConfigurationFactory->create($this->metadata, $request);
        $quote = $quoteContext->getQuote();

        /** @var QuoteItemInterface $quoteItem */
        $quoteItem = $this->newResourceFactory->create($configuration, $this->factory);

        $form = $formFactory->create(
            $configuration->getFormType(),
            $addToQuoteCommandFactory->createWithQuoteAndQuoteItem($quote, $quoteItem),
            $configuration->getFormOptions()
        );

        if ($request->isMethod('POST') && $form->handleRequest($request)->isValid()) {
            /** @var AddToQuoteCommandInterface $addToQuoteCommand */
            $addToQuoteCommand = $form->getData();

            $event = $this->eventDispatcher->dispatchPreEvent(QuoteActionEnum::ADD, $configuration, $quoteItem);
            if ($event->isStopped() && !$configuration->isHtmlRequest()) {
                throw new HttpException($event->getErrorCode(), $event->getMessage());
            }
            if ($event->isStopped()) {
                $this->flashHelper->addFlashFromEvent($configuration, $event);

                return $this->redirectHandler->redirectToIndex($configuration, $quoteItem);
            }

            $quoteModifier->addToQuote($quote, $addToQuoteCommand->getCartItem());

            $entityManager->persist($quote);
            $entityManager->flush();

            $resourceControllerEvent = $this->eventDispatcher->dispatchPostEvent(QuoteActionEnum::ADD, $configuration, $quoteItem);
            if ($resourceControllerEvent->hasResponse()) {
                return $resourceControllerEvent->getResponse();
            }

            $this->flashHelper->addSuccessFlash($configuration, QuoteActionEnum::ADD, $quoteItem);

            if ($request->isXmlHttpRequest()) {
                return $this->viewHandler->handle($configuration, View::create([], Response::HTTP_CREATED));
            }
        }

        return $this->redirectHandler->redirectToResource($configuration, $quoteItem);
    }

    public function removeAction(
        Request $request,
        QuoteContextInterface $quoteContext,
        QuoteModifierInterface $quoteModifier,
        EntityManagerInterface $entityManager
    ): Response {
        $configuration = $this->requestConfigurationFactory->create($this->metadata, $request);

        $this->isGrantedOr403($configuration, QuoteActionEnum::REMOVE);
        /** @var QuoteItemInterface $quoteItem */
        $quoteItem = $this->findOr404($configuration);

        $event = $this->eventDispatcher->dispatchPreEvent(QuoteActionEnum::REMOVE, $configuration, $quoteItem);

        if ($configuration->isCsrfProtectionEnabled() && !$this->isCsrfTokenValid((string) $quoteItem->getId(), (string) $request->request->get('_csrf_token'))) {
            throw new HttpException(Response::HTTP_FORBIDDEN, 'Invalid csrf token.');
        }

        if ($event->isStopped() && !$configuration->isHtmlRequest()) {
            throw new HttpException($event->getErrorCode(), $event->getMessage());
        }
        if ($event->isStopped()) {
            $this->flashHelper->addFlashFromEvent($configuration, $event);

            return $this->redirectHandler->redirectToIndex($configuration, $quoteItem);
        }

        $quote = $quoteContext->getQuote();
        if ($quote !== $quoteItem->getQuote()) {
            $this->addFlash('error', $this->get('translator')->trans('dedi_quote.quote.cannot_modify', [], 'flashes'));

            if (!$configuration->isHtmlRequest()) {
                return $this->viewHandler->handle($configuration, View::create(null, Response::HTTP_NO_CONTENT));
            }

            return $this->redirectHandler->redirectToIndex($configuration, $quoteItem);
        }

        $quoteModifier->removeFromQuote($quote, $quoteItem);

        $this->repository->remove($quoteItem);

        $entityManager->persist($quote);
        $entityManager->flush();

        $this->eventDispatcher->dispatchPostEvent(QuoteActionEnum::REMOVE, $configuration, $quoteItem);

        if (!$configuration->isHtmlRequest()) {
            return $this->viewHandler->handle($configuration, View::create(null, Response::HTTP_NO_CONTENT));
        }

        $this->flashHelper->addSuccessFlash($configuration, QuoteActionEnum::REMOVE, $quoteItem);

        return $this->redirectHandler->redirectToIndex($configuration, $quoteItem);
    }
}
